from __future__ import annotations

import datetime
import json
from decimal import Decimal

from pretix_fattura_elettronica.serializers import InvoiceSerializer

import pytest
import time_machine
from django_countries.fields import Country
from pretix.base.models import (
    Event,
    Invoice,
    InvoiceAddress,
    Order,
    OrderPosition,
    TaxRule,
)
from pretix.base.services.invoices import build_invoice, generate_cancellation

pytestmark = pytest.mark.django_db


@pytest.fixture
def tax_rule_ticket(event):
    return event.tax_rules.create(rate=Decimal("19.00"))


@pytest.fixture
def tax_rule_hotel(event):
    return event.tax_rules.create(rate=Decimal("0.00"), name="N1")


@pytest.fixture
def tax_rule_membership(event):
    return event.tax_rules.create(rate=Decimal("19.00"), name="N2.2")


@pytest.fixture
@time_machine.travel("2017-12-01T10:00:00Z")
def private_order(
    event: Event,
    tax_rule_ticket: TaxRule,
    tax_rule_hotel: TaxRule,
    tax_rule_membership: TaxRule,
):
    codice_destinatario = "000000"
    pec_address = "ciccio@gmail.com"

    now = datetime.datetime.now(datetime.timezone.utc)

    o = Order.objects.create(
        code="FOO",
        event=event,
        email="dummy@dummy.test",
        status=Order.STATUS_PENDING,
        secret="k24fiuwvu8kxz3y1",
        datetime=now,
        expires=now + datetime.timedelta(days=9),
        total=23,
        locale="en",
        meta_info=json.dumps(
            {
                "email": pec_address,
                "pec": pec_address,
                "codice_fiscale": "cod_fiscale",
                "sdi": codice_destinatario,
            }
        ),
    )

    InvoiceAddress.objects.create(
        order=o,
        company="",
        name_parts={
            "_scheme": "given_family",
            "given_name": "John",
            "family_name": "Doe",
        },
        country=Country("IT"),
        city="Napoli",  # mandatory
        zipcode="01020",  # mandatory
        vat_id="",
        vat_id_validated=True,
        is_business=False,
    )

    # Ticket

    ticket_item = event.items.create(
        name="Ticket", default_price=23, tax_rule=tax_rule_ticket
    )

    OrderPosition.objects.create(
        order=o,
        item=ticket_item,
        variation=None,
        price=Decimal("23"),
        attendee_name_parts={"full_name": "Peter", "_scheme": "full"},
        secret="z3fsn8jyufm5kpk768q69gkbyr5f4h6w",
        pseudonymization_id="ABCDEFGHKL",
        positionid=1,
    )

    # Membership

    membership_item = event.items.create(
        name="Membership", default_price=10, tax_rule=tax_rule_membership
    )

    OrderPosition.objects.create(
        order=o,
        item=membership_item,
        variation=None,
        price=Decimal("10.00"),
        attendee_name_parts={"full_name": "Peter", "_scheme": "full"},
        secret="z3fsn8jyufm5kpk768q69gkbyr5f4h6w",
        pseudonymization_id="SomethingElse",
        positionid=2,
    )

    # Hotel

    hotel_item = event.items.create(
        name="Hotel", default_price=100, tax_rule=tax_rule_hotel
    )

    OrderPosition.objects.create(
        order=o,
        item=hotel_item,
        variation=None,
        price=Decimal("100.00"),
        attendee_name_parts={"full_name": "Peter", "_scheme": "full"},
        secret="z3fsn8jyufm5kpk768q69gkbyr5f4h6w",
        pseudonymization_id="SomethingElseHotel",
        positionid=3,
    )

    return o


@pytest.fixture
def private_invoice(private_order: Order):
    invoice = Invoice(
        order=private_order,
        event=private_order.event,
        organizer=private_order.event.organizer,
        date=datetime.datetime.now(private_order.event.timezone).date(),
    )
    invoice.event.settings["invoice_address_from"] = "Via Roma 11"
    invoice.event.settings["invoice_address_from_name"] = "Python Italia APS"
    invoice.event.settings["invoice_address_from_zipcode"] = "12345"
    invoice.event.settings["invoice_address_from_city"] = "Firenze"
    invoice.event.settings["invoice_address_from_country"] = str(Country("IT"))
    invoice.event.settings["invoice_address_from_vat_id"] = "02053290630"
    invoice.save()
    invoice = build_invoice(invoice)
    if private_order.status == Order.STATUS_CANCELED:
        generate_cancellation(invoice, False)

    return invoice


@pytest.fixture
def serialized_invoice(private_invoice: Invoice):
    return InvoiceSerializer.serialize(private_invoice).model_dump()


def test_header(serialized_invoice: dict):
    assert serialized_invoice["fattura_elettronica_header"] == {
        "dati_trasmissione": {
            "id_trasmittente": {"id_paese": "IT", "id_codice": "02053290630"},
            "progressivo_invio": "DUMMY-00001",
            "formato_trasmissione": "FPR12",
            "codice_destinatario": "000000",
            "pec_destinatario": "ciccio@gmail.com",
        },
        "cedente_prestatore": {
            "dati_anagrafici": {
                "id_fiscale_iva": {"id_paese": "IT", "id_codice": "02053290630"},
                "codice_fiscale": "94144670489",
                "anagrafica": {
                    "denominazione": "Python Italia APS",
                    "nome": None,
                    "cognome": None,
                    "titolo": None,
                    "cod_eori": None,
                },
                "regime_fiscale": "RF01",
            },
            "sede": {
                "indirizzo": "Via Roma 11",
                "numero_civico": None,
                "cap": "12345",
                "comune": "Firenze",
                "provincia": None,
                "nazione": "IT",
            },
            "contatti": {"telefono": None, "fax": None, "email": "info@python.it"},
        },
        "cessionario_committente": {
            "dati_anagrafici": {
                "id_fiscale_iva": None,
                "codice_fiscale": "COD_FISCALE",
                "anagrafica": {
                    "denominazione": None,
                    "nome": "John",
                    "cognome": "Doe",
                    "titolo": None,
                    "cod_eori": None,
                },
            },
            "sede": {
                "indirizzo": "John Doe\n01020 Napoli\nItaly",
                "numero_civico": None,
                "cap": "01020",
                "comune": "Napoli",
                "provincia": None,
                "nazione": "IT",
            },
        },
    }


def test_body(serialized_invoice: dict):
    assert serialized_invoice["fattura_elettronica_body"] == [
        {
            "dati_generali": {
                "dati_generali_documento": {
                    "tipo_documento": "TD01",
                    "divisa": "EUR",
                    "data": "2024-03-18",
                    "numero": "DUMMY-00001",
                }
            },
            "dati_beni_servizi": {
                "dettaglio_linee": [
                    {
                        "numero_linea": 1,
                        "descrizione": "Ticket<br />Attendee: Peter",
                        "quantita": "1.00",
                        "unita_misura": None,
                        "prezzo_unitario": "19.33",
                        "prezzo_totale": "19.33",
                        "aliquota_iva": "19.00",
                        "ritenuta": None,
                        "natura": None,
                    },
                    {
                        "numero_linea": 2,
                        "descrizione": "Membership<br />Attendee: Peter",
                        "quantita": "1.00",
                        "unita_misura": None,
                        "prezzo_unitario": "8.40",
                        "prezzo_totale": "8.40",
                        "aliquota_iva": "19.00",
                        "ritenuta": None,
                        "natura": "N2.2",
                    },
                    {
                        "numero_linea": 3,
                        "descrizione": "Hotel<br />Attendee: Peter",
                        "quantita": "1.00",
                        "unita_misura": None,
                        "prezzo_unitario": "100.00",
                        "prezzo_totale": "100.00",
                        "aliquota_iva": "0.00",
                        "ritenuta": None,
                        "natura": "N1",
                    },
                ],
                "dati_riepilogo": [
                    {
                        "aliquota_iva": "19.00",
                        "natura": None,
                        "imponibile_importo": "19.33",
                        "imposta": "3.67",
                    },
                    {
                        "aliquota_iva": "19.00",
                        "natura": "N2.2",
                        "imponibile_importo": "8.40",
                        "imposta": "1.60",
                    },
                    {
                        "aliquota_iva": "0.00",
                        "natura": "N1",
                        "imponibile_importo": "100.00",
                        "imposta": "0.00",
                    },
                ],
            },
        }
    ]
